@@ -1,5 +1,5 @@
 /* This file is part of the 'stringi' package for R.
- * Copyright (c) 2013-2017, Marek Gagolewski and other contributors.
+ * Copyright (c) 2013-2019, Marek Gagolewski and other contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -61,6 +61,9 @@
  *
  * @version 0.4-1 (Marek Gagolewski, 2014-12-08)
  *    #23: add `overlap` option
+ *
+ * @version 1.1.6 (Marek Gagolewski, 2017-11-10)
+ *    PROTECT STRING_ELT(names, i)
  */
 UCollator* stri__ucol_open(SEXP opts_collator)
 {
@@ -76,7 +79,7 @@ UCollator* stri__ucol_open(SEXP opts_collator)
       return col;
    }
 
-   SEXP names = Rf_getAttrib(opts_collator, R_NamesSymbol);
+   SEXP names = PROTECT(Rf_getAttrib(opts_collator, R_NamesSymbol));
    if (names == R_NilValue || LENGTH(names) != narg)
       Rf_error(MSG__INCORRECT_COLLATOR_OPTION_SPEC); // error() allowed here
 
@@ -96,42 +99,49 @@ UCollator* stri__ucol_open(SEXP opts_collator)
       if (STRING_ELT(names, i) == NA_STRING)
          Rf_error(MSG__INCORRECT_COLLATOR_OPTION_SPEC); // error() allowed here
 
-      const char* curname = CHAR(STRING_ELT(names, i));
+      SEXP tmp_arg;
+      PROTECT(tmp_arg = STRING_ELT(names, i));
+      const char* curname = stri__copy_string_Ralloc(tmp_arg, "curname");  /* this is R_alloc'ed */
+      UNPROTECT(1);
+
+      PROTECT(tmp_arg = VECTOR_ELT(opts_collator, i));
       if (!strcmp(curname, "locale")) {
-         opt_LOCALE = stri__prepare_arg_locale(VECTOR_ELT(opts_collator, i), "locale", true); /* this is R_alloc'ed */
+         opt_LOCALE = stri__prepare_arg_locale(tmp_arg, "locale", true); /* this is R_alloc'ed */
       } else if  (!strcmp(curname, "strength")) {
-         int val = stri__prepare_arg_integer_1_notNA(VECTOR_ELT(opts_collator, i), "strength");
+         int val = stri__prepare_arg_integer_1_notNA(tmp_arg, "strength");
          if (val < (int)UCOL_PRIMARY + 1) val = (int)UCOL_PRIMARY + 1;
          else if (val > (int)UCOL_STRENGTH_LIMIT + 1) val = (int)UCOL_STRENGTH_LIMIT + 1;
          opt_STRENGTH = (UColAttributeValue)(val-1);
 //      } else if  (!strcmp(curname, "overlap") && allow_overlap) {
-//         bool val_bool = stri__prepare_arg_logical_1_notNA(VECTOR_ELT(opts_collator, i), "overlap");
+//         bool val_bool = stri__prepare_arg_logical_1_notNA(tmp_arg, "overlap");
 //         opt_OVERLAP = (val_bool?USEARCH_ON:USEARCH_OFF);
       } else if  (!strcmp(curname, "alternate_shifted")) {
-         bool val_bool = stri__prepare_arg_logical_1_notNA(VECTOR_ELT(opts_collator, i), "alternate_shifted");
+         bool val_bool = stri__prepare_arg_logical_1_notNA(tmp_arg, "alternate_shifted");
          opt_ALTERNATE_HANDLING = (val_bool?UCOL_SHIFTED:UCOL_NON_IGNORABLE);
       } else if  (!strcmp(curname, "uppercase_first")) {
          SEXP val;
-         PROTECT(val = stri_prepare_arg_logical_1(VECTOR_ELT(opts_collator, i), "uppercase_first"));
+         PROTECT(val = stri_prepare_arg_logical_1(tmp_arg, "uppercase_first"));
          opt_CASE_FIRST = (LOGICAL(val)[0]==NA_LOGICAL?UCOL_OFF:
                           (LOGICAL(val)[0]?UCOL_UPPER_FIRST:UCOL_LOWER_FIRST));
          UNPROTECT(1);
       } else if  (!strcmp(curname, "french")) {
-         bool val_bool = stri__prepare_arg_logical_1_notNA(VECTOR_ELT(opts_collator, i), "french");
+         bool val_bool = stri__prepare_arg_logical_1_notNA(tmp_arg, "french");
          opt_FRENCH_COLLATION = (val_bool?UCOL_ON:UCOL_OFF);
       } else if  (!strcmp(curname, "case_level")) {
-         bool val_bool = stri__prepare_arg_logical_1_notNA(VECTOR_ELT(opts_collator, i), "case_level");
+         bool val_bool = stri__prepare_arg_logical_1_notNA(tmp_arg, "case_level");
          opt_CASE_LEVEL = (val_bool?UCOL_ON:UCOL_OFF);
       } else if  (!strcmp(curname, "normalization")) {
-         bool val_bool = stri__prepare_arg_logical_1_notNA(VECTOR_ELT(opts_collator, i), "normalization");
+         bool val_bool = stri__prepare_arg_logical_1_notNA(tmp_arg, "normalization");
          opt_NORMALIZATION_MODE = (val_bool?UCOL_ON:UCOL_OFF);
       } else if  (!strcmp(curname, "numeric")) {
-         bool val_bool = stri__prepare_arg_logical_1_notNA(VECTOR_ELT(opts_collator, i), "numeric");
+         bool val_bool = stri__prepare_arg_logical_1_notNA(tmp_arg, "numeric");
          opt_NUMERIC_COLLATION = (val_bool?UCOL_ON:UCOL_OFF);
       } else {
          Rf_warning(MSG__INCORRECT_COLLATOR_OPTION, curname);
       }
+      UNPROTECT(1);
    }
+   UNPROTECT(1); /* names */
 
    // create collator
    UErrorCode status = U_ZERO_ERROR;
